Class: KitchenHooks::App

Inherits:
Sinatra::Application
  • Object
show all
Defined in:
lib/kitchen_hooks/app.rb,
lib/kitchen_hooks/helpers.rb

Class Method Summary collapse

Instance Method Summary collapse

Class Method Details

.apply_constraints(constraints, environment, knife) ⇒ Object



417
418
419
420
421
422
423
424
425
426
427
428
429
# File 'lib/kitchen_hooks/helpers.rb', line 417

def self.apply_constraints constraints, environment, knife
  # Ripped from Berkshelf::Cli::apply and Berkshelf::Lockfile::apply
  # https://github.com/berkshelf/berkshelf/blob/master/lib/berkshelf/cli.rb
  # https://github.com/berkshelf/berkshelf/blob/master/lib/berkshelf/lockfile.rb
  $stdout.puts 'started apply_constraints environment=%s, knife=%s' % [
    environment.inspect, knife.inspect
  ]
  chef_environment = get_environment environment, knife
  raise 'Could not find environment "%s"' % environment if chef_environment.nil?
  chef_environment.cookbook_versions = constraints
  chef_environment.save
  $stdout.puts 'finished apply_constraints: %s' % environment
end

.author(event) ⇒ Object



546
547
548
# File 'lib/kitchen_hooks/helpers.rb', line 546

def self.author event
  event['user_name']
end

.backlog!Object



37
38
39
40
41
42
43
44
45
# File 'lib/kitchen_hooks/app.rb', line 37

def self.backlog!
  @@backlog = Queue.new
  @@backlog_worker = Thread.new do
    loop do
      event = @@backlog.shift
      App.process event
    end
  end
end

.berks_install(berksfile, knife = nil) ⇒ Object



297
298
299
300
301
302
303
304
305
306
307
308
309
310
311
312
313
314
315
316
317
318
319
320
321
322
# File 'lib/kitchen_hooks/helpers.rb', line 297

def self.berks_install berksfile, knife=nil
  $stdout.puts 'started berks_install berksfile=%s' % berksfile.inspect
  env_git_dir = ENV.delete 'GIT_DIR'
  env_git_work_tree = ENV.delete 'GIT_WORK_TREE'

  knife_args = if knife
                 '--config %s' % Shellwords::escape(berkshelf_config knife)
               end

  cmd = 'berks install --debug --berksfile %s %s 2>&1' % [
    Shellwords::escape(berksfile), knife_args
  ]
  begin
    $stdout.puts "berks_install: %s" % cmd
    out = `#{cmd}`
    raise out unless $?.exitstatus.zero?
  rescue
    raise 'Could not perform berks_install with config %s' % [
      berksfile.inspect
    ]
  end

  ENV['GIT_DIR'] = env_git_dir
  ENV['GIT_WORK_TREE'] = env_git_work_tree
  $stdout.puts 'finished berks_install: %s' % berksfile
end

.berks_upload(berksfile, knife, options = {}) ⇒ Object



325
326
327
328
329
330
331
332
333
334
335
336
337
338
339
340
341
342
343
344
345
346
347
# File 'lib/kitchen_hooks/helpers.rb', line 325

def self.berks_upload berksfile, knife, options={}
  $stdout.puts 'started berks_upload berksfile=%s, knife=%s' % [
    berksfile.inspect, knife.inspect
  ]
  config_path = berkshelf_config(knife)

  cmd = 'berks upload --no-ssl-verify --debug --berksfile %s --config %s 2>&1' % [
    Shellwords::escape(berksfile), Shellwords::escape(config_path)
  ]

  begin
    $stdout.puts "berks_upload: %s" % cmd
    out = `#{cmd}`
    raise out unless $?.exitstatus.zero?
  rescue
    raise 'Could not perform berks_upload with config %s, knife %s' % [
      berksfile.inspect, knife.inspect
    ]
  end

  FileUtils.rm_rf config_path
  $stdout.puts 'finished berks_upload: %s' % berksfile
end

.berkshelf_config(knife) ⇒ Object



276
277
278
279
280
281
282
283
284
285
286
287
288
289
290
291
292
293
294
# File 'lib/kitchen_hooks/helpers.rb', line 276

def self.berkshelf_config knife
  ridley = Ridley::from_chef_config knife, ssl: { verify: false }
  config = {
    chef: {
      node_name: ridley.client_name,
      client_key: ridley.client_key,
      chef_server_url: ridley.server_url
    },
    ssl: {
      verify: false
    }
  }
  $stdout.puts 'Berkshelf config: %s' % JSON::pretty_generate(config)
  config_path = File.join tmp, "#{SecureRandom.hex}-berkshelf.json"
  File.open(config_path, 'w') do |f|
    f.puts JSON::generate(config)
  end
  return config_path
end

.close!Object



30
31
32
33
34
35
# File 'lib/kitchen_hooks/app.rb', line 30

def self.close!
  @@sync_worker.kill if defined? @@sync_worker
  @@backlog_worker.kill
  @@db.flush
  @@db.close
end

.commit_to_data_bags?(event) ⇒ Boolean

Returns:

  • (Boolean)


653
654
655
# File 'lib/kitchen_hooks/helpers.rb', line 653

def self.commit_to_data_bags? event
  data_bag_repo?(event) && not_deleted?(event)
end

.commit_to_environments?(event) ⇒ Boolean

Returns:

  • (Boolean)


648
649
650
# File 'lib/kitchen_hooks/helpers.rb', line 648

def self.commit_to_environments? event
  environment_repo?(event) && not_deleted?(event)
end

.commit_to_kitchen?(event) ⇒ Boolean

Returns:

  • (Boolean)


638
639
640
# File 'lib/kitchen_hooks/helpers.rb', line 638

def self.commit_to_kitchen? event
  repo_name(event) == 'kitchen' && not_deleted?(event)
end

.commit_to_master?(event) ⇒ Boolean

Returns:

  • (Boolean)


628
629
630
# File 'lib/kitchen_hooks/helpers.rb', line 628

def self.commit_to_master? event
  event['ref'] == 'refs/heads/master'
end

.commit_to_realm?(event) ⇒ Boolean

Returns:

  • (Boolean)


658
659
660
# File 'lib/kitchen_hooks/helpers.rb', line 658

def self.commit_to_realm? event
  repo_name(event) =~ /^realm_/
end

.commit_to_roles?(event) ⇒ Boolean

Returns:

  • (Boolean)


643
644
645
# File 'lib/kitchen_hooks/helpers.rb', line 643

def self.commit_to_roles? event
  role_repo?(event) && not_deleted?(event)
end

.config!(config) ⇒ Object



58
59
60
61
62
63
64
65
66
67
68
69
70
71
72
# File 'lib/kitchen_hooks/app.rb', line 58

def self.config! config
  $stderr.puts 'ENV: %s' % JSON::pretty_generate(ENV.to_hash)
  @@config = config
  @@hipchat = nil
  if config['hipchat']
    @@hipchat = HipChat::Client.new config['hipchat']['token']
    @@hipchat_nick = config['hipchat']['nick'] || raise('No HipChat "nick" provided')
    @@hipchat_room = config['hipchat']['room'] || raise('No HipChat "room" provided')
  end
  @@git_protocol = config.fetch('git_protocol', 'daemon')
  @@knives = config['knives'].map do |_, knife|
    Pathname.new(knife).expand_path.realpath.to_s
  end
  @@sync_interval = config['sync_interval']
end

.cookbook_name(event) ⇒ Object



560
561
562
# File 'lib/kitchen_hooks/helpers.rb', line 560

def self.cookbook_name event
  repo_name(event).sub(/^(app|base|realm|fork)_/, 'bjn_')
end

.cookbook_repo?(event) ⇒ Boolean

Returns:

  • (Boolean)


565
566
567
568
569
# File 'lib/kitchen_hooks/helpers.rb', line 565

def self.cookbook_repo? event
  repo_name(event) =~ /^(app|base|realm|fork)_/ ||
    repo_name(event) =~ /cookbook/ ||
    repo_namespace(event) =~ /cookbook/
end

.data_bag_from_file(data_bag, items, knives) ⇒ Object



235
236
237
238
239
240
241
242
243
244
245
246
247
248
249
# File 'lib/kitchen_hooks/helpers.rb', line 235

def self.data_bag_from_file data_bag, items, knives
  $stdout.puts 'Uploading data_bags'
  begin
    items.each do |item|
      # Try to guess if there is one repo per data bag or
      # all data bags are in the sub-folders in the same repo.
      if item.split('/').length > 1
        data_bag = item.split('/')[-2]
      end
      with_each_knife_do 'data bag from file ' + data_bag + ' ' + item, knives
    end
  rescue
    raise 'Could not upload data bags'
  end
end

.data_bag_repo?(event) ⇒ Boolean

Returns:

  • (Boolean)


572
573
574
575
# File 'lib/kitchen_hooks/helpers.rb', line 572

def self.data_bag_repo? event
  repo_name(event) =~ /data.*bag/ ||
    repo_namespace(event) =~ /data.*bag/
end

.db!(path = @@db_path) ⇒ Object



19
20
21
22
23
24
25
26
# File 'lib/kitchen_hooks/app.rb', line 19

def self.db! path=@@db_path
  if defined? @@db
    @@db.flush
    @@db.close
  end
  @@db_path = path
  @@db = Daybreak::DB.new path
end

.environment_from_file(environments, knives) ⇒ Object



264
265
266
267
268
269
270
271
272
273
# File 'lib/kitchen_hooks/helpers.rb', line 264

def self.environment_from_file environments, knives
  $stdout.puts 'Uploading environments'
  begin
    environments.each do |environment|
      with_each_knife_do 'environment from file ' + environment, knives
    end
  rescue
    raise 'Could not upload environments'
  end
end

.environment_repo?(event) ⇒ Boolean

Returns:

  • (Boolean)


578
579
580
581
# File 'lib/kitchen_hooks/helpers.rb', line 578

def self.environment_repo? event
  repo_name(event) =~ /environment/ ||
    repo_namespace(event) =~ /environment/
end

.files_pushed(event) ⇒ Object



537
538
539
540
541
542
543
# File 'lib/kitchen_hooks/helpers.rb', line 537

def self.files_pushed event
  files = []
  event.fetch('commits').each do |commit|
    files << commit.select { |k,v| k =~ /(added|modified)/ }.values.flatten
  end
  files.flatten.uniq
end

.generic_details(event) ⇒ Object



518
519
520
521
522
523
# File 'lib/kitchen_hooks/helpers.rb', line 518

def self.generic_details event
  return if event.nil?
  %Q|
    <i>#{author(event)}</i> pushed #{push_details(event)}
    |.strip
end

.get_environment(environment, knife) ⇒ Object



399
400
401
402
# File 'lib/kitchen_hooks/helpers.rb', line 399

def self.get_environment environment, knife
  ridley = Ridley::from_chef_config knife, ssl: { verify: false }
  ridley.environment.find environment
end

.git_daemon_style_url(event) ⇒ Object



595
596
597
# File 'lib/kitchen_hooks/helpers.rb', line 595

def self.git_daemon_style_url event
  event['repository']['url'].sub(':', '/').sub('@', '://')
end

.gitlab_tag_url(event) ⇒ Object



606
607
608
609
# File 'lib/kitchen_hooks/helpers.rb', line 606

def self.gitlab_tag_url event
  url = git_daemon_style_url(event).sub(/^git/, 'http').sub(/\.git$/, '')
  "#{url}/commits/#{tag_name(event)}"
end

.gitlab_url(event) ⇒ Object



600
601
602
603
# File 'lib/kitchen_hooks/helpers.rb', line 600

def self.gitlab_url event
  url = git_daemon_style_url(event).sub(/^git/, 'http').sub(/\.git$/, '')
  "#{url}/commit/#{event['after']}"
end

.humanize_seconds(secs) ⇒ Object



32
33
34
35
36
37
38
39
40
41
42
43
44
# File 'lib/kitchen_hooks/helpers.rb', line 32

def self.humanize_seconds secs
  [
    [ 60, :seconds ],
    [ 60, :minutes ],
    [ 24, :hours ],
    [ 1000, :days ]
  ].map { |count, name|
    if secs > 0
      secs, n = secs.divmod(count)
      "#{n.to_i} #{name}"
    end
  }.compact.reverse.join(' ')
end

.kitchen_upload(knives) ⇒ Object



200
201
202
203
204
205
206
207
208
209
210
211
212
213
214
215
216
217
218
219
220
221
222
223
224
225
226
227
228
229
230
231
232
# File 'lib/kitchen_hooks/helpers.rb', line 200

def self.kitchen_upload knives
  if Dir.exist? 'data_bags'
    $stdout.puts 'Uploading data_bags'
    begin
      with_each_knife_do 'upload data_bags --chef-repo-path .', knives
    rescue
      raise 'Could not upload data bags'
    end
  end

  if Dir.exist? 'roles'
    $stdout.puts 'Uploading roles'
    begin
      with_each_knife_do 'upload roles --chef-repo-path .', knives
    rescue
      raise 'Could not upload roles'
    end
  end

  if Dir.exist? 'environments'
    $stdout.puts 'Uploading environments'
    knives.peach do |k|
      begin
        Dir['environments/*'].each do |e|
          # Can't use the default logic, as we maintain our own pins
          upload_environment e, k
        end
      rescue
        raise 'Could not upload environments'
      end
    end
  end
end

.latest_commit(event) ⇒ Object



612
613
614
# File 'lib/kitchen_hooks/helpers.rb', line 612

def self.latest_commit event
  event['commits'].last['id']
end

.lockfile_constraints(lockfile_path) ⇒ Object



432
433
434
435
436
437
438
439
440
441
442
443
# File 'lib/kitchen_hooks/helpers.rb', line 432

def self.lockfile_constraints lockfile_path
  # Ripped from Berkshelf::Cli::apply and Berkshelf::Lockfile::apply
  # https://github.com/berkshelf/berkshelf/blob/master/lib/berkshelf/cli.rb
  # https://github.com/berkshelf/berkshelf/blob/master/lib/berkshelf/lockfile.rb
  lockfile = Berkshelf::Lockfile.from_file lockfile_path
  constraints = lockfile.graph.locks.inject({}) do |hash, (name, dependency)|
    hash[name] = "= #{dependency.locked_version.to_s}"
    hash
  end
  $stdout.puts 'constraints: %s -> %s' % [ lockfile_path, constraints ]
  return constraints
end

.not_deleted?(event) ⇒ Boolean

Returns:

  • (Boolean)


633
634
635
# File 'lib/kitchen_hooks/helpers.rb', line 633

def self.not_deleted? event
  event['after'] != '0000000000000000000000000000000000000000'
end

.notification(entry) ⇒ Object



481
482
483
484
485
486
487
488
489
490
491
492
493
494
495
496
497
498
499
500
501
502
503
504
505
506
507
508
509
510
511
512
513
# File 'lib/kitchen_hooks/helpers.rb', line 481

def self.notification entry
  return entry[:error] if entry[:error]
  event = entry[:event]

  case entry[:type]
  when 'synced', 'unsynced'
    if event.is_a? String
      event
    else
      num_deletions = event[:num_deletions].to_i
      deletions = '%s, ' % pluralize(num_deletions, 'deletion')
      deletions = '' unless num_deletions > 0
      'Synced <b>%d</b> of <b>%d</b> nodes (%s%s elapsed)' % [
        event[:num_successes],
        event[:num_nodes],
        deletions,
        humanize_seconds(event[:elapsed])
      ]
    end
  when 'kitchen upload'
    %Q| <i>#{author(event)}</i> updated <a href="#{gitlab_url(event)}">the Kitchen</a> |
  when 'cookbook upload'
    %Q| <i>#{author(event)}</i> released <a href="#{gitlab_tag_url(event)}">#{tag_name(event)}</a> of <a href="#{gitlab_url(event)}">#{cookbook_name(event)}</a> |
  when 'constraint application'
    %Q| <i>#{author(event)}</i> constrained <a href="#{gitlab_tag_url(event)}">#{tag_name(event)}</a> with <a href="#{gitlab_url(event)}">#{cookbook_name(event)}</a> #{version_link event} |
  when 'release'
    %Q| Kitchen Hooks <b>v#{event}</b> released! |
  when 'upload from files'
    %Q| <i>#{author(event)}</i> uploaded <a href="#{gitlab_url(event)}">files</a> to chef |
  else
    raise entry.inspect
  end.strip
end

.perform_constraint_application(event, knives) ⇒ Object



56
57
58
59
60
61
62
63
64
65
66
67
68
69
70
71
72
73
74
75
76
77
78
79
80
81
82
83
84
85
86
87
88
89
90
91
92
93
94
95
96
# File 'lib/kitchen_hooks/helpers.rb', line 56

def self.perform_constraint_application event, knives
  $stdout.puts 'started perform_constraint_application event=%s, knives=%s' % [
    event['after'], knives.inspect
  ]

  tmp_clone event, :tagged_commit do |clone|
    Dir.chdir clone do

      $stdout.puts 'Applying constraints'
      constraints = lockfile_constraints 'Berksfile.lock'
      environment = tag_name event

      environments = Dir['environments/*.json'].map { |f| File.basename f, '.json' }
      unless environments.include?(environment)
        $stderr.puts 'WARNING: No local environment: %s' % environment
        next
      end

      knives.peach do |k|
        apply_constraints constraints, environment, k
        verify_constraints constraints, environment, k
      end

      rev  = `git rev-list -1 #{environment}`.strip
      refs = `git show-ref --tags -d | grep '^#{rev}'`.lines.map(&:strip)
      tags = refs.map do |r|
        $1 if r =~ %r|refs/tags/(?<tag>.*?)\^|s
      end

      version_tag = tags.select { |t| t =~ /^v\d+/ }.shift
      event['version'] = version_tag
    end
  end

  $stdout.puts "finished perform_constraint_application: #{event['after']}"
  return # no error

rescue Git::GitExecuteError
  $stderr.puts 'WARNING: Could not check out tagged commit'
  return false
end

.perform_cookbook_upload(event, knives) ⇒ Object



116
117
118
119
120
121
122
123
124
125
126
127
128
129
130
131
132
133
134
135
136
137
138
139
140
141
142
143
144
145
146
147
148
149
150
151
152
153
154
155
156
157
158
159
160
161
162
163
164
165
166
167
168
169
170
171
# File 'lib/kitchen_hooks/helpers.rb', line 116

def self.perform_cookbook_upload event, knives
  $stdout.puts 'started perform_cookbook_upload event=%s, knives=%s' % [
    event['after'], knives.inspect
  ]

  tmp_clone event, :tagged_commit do |clone|
    tagged_version = tag_name(event).delete('v')
    if File.exist? File.join(clone, 'VERSION')
      cookbook_version = File.read(File.join(clone, 'VERSION')).strip
    else
      cookbook_version = File.foreach(File.join(clone, 'metadata.rb')).grep(/version/)[0][/\"(.*)\"/,1]
    end
    unless tagged_version == cookbook_version
      raise 'Tagged version does not match cookbook version'
    end

    berksfile = File.join clone, 'Berksfile'

    if File.exist? berksfile
      $stdout.puts 'Uploading dependencies'
      FileUtils.rm_rf File.join(ENV['HOME'], '.berkshelf')
      berks_install berksfile

      knives.map do |knife|
        tmp_root = Dir.mktmpdir
        tmp_path = File.join tmp_root, File.basename(knife, '.rb')
        FileUtils.copy_entry clone, tmp_path
        [ knife, tmp_path, tmp_root ]
      end.peach do |(knife, tmp_path, tmp_root)|
        tmp_berksfile = File.join tmp_path, 'Berksfile'
        berks_upload tmp_berksfile, knife
        FileUtils.rm_rf tmp_root
      end
    end

    Dir.chdir clone do
      # N.B. Knife uploading the cookbook should be unnecessary thanks to Berkshelf...
      # $stdout.puts 'Uploading cookbook'
      # begin
      #   with_each_knife_do "cookbook upload #{cookbook_name event} -o .. --freeze", knives
      # rescue => e
      #   unless e.to_s =~ /frozen/i # Ignore frozen cookbooks already uploaded
      #     raise "Knife exited unsuccessfully: #{e}"
      #   end
      # end

      if commit_to_realm? event
        $stdout.puts 'Uploading bundled roles, environments, and data bags'
        kitchen_upload knives
      end
    end
  end

  $stdout.puts "finished cookbook_upload: #{event['after']}"
  return # no error
end

.perform_kitchen_upload(event, knives) ⇒ Object



99
100
101
102
103
104
105
106
107
108
109
110
111
112
113
# File 'lib/kitchen_hooks/helpers.rb', line 99

def self.perform_kitchen_upload event, knives
  return false unless commit_to_master?(event)
  $stdout.puts 'started perform_kitchen_upload event=%s, knives=%s' % [
    event['after'], knives.inspect
  ]

  tmp_clone event, :latest_commit do |clone|
    Dir.chdir clone do
      kitchen_upload knives
    end
  end

  $stdout.puts "finished perform_kitchen_upload: #{event['after']}"
  return # no error
end

.perform_upload_from_file(event, knives) ⇒ Object



174
175
176
177
178
179
180
181
182
183
184
185
186
187
188
189
190
191
192
193
194
195
196
# File 'lib/kitchen_hooks/helpers.rb', line 174

def self.perform_upload_from_file event, knives
  return false unless commit_to_master?(event)
  $stdout.puts 'started perform_upload_from_file event=%s, knives=%s' % [
    event['after'], knives.inspect
  ]

  tmp_clone event, :latest_commit do |clone|
    Dir.chdir clone do
      if commit_to_data_bags?(event)
        data_bag_from_file repo_name(event), files_pushed(event), knives
      end
      if commit_to_environments?(event)
        environment_from_file files_pushed(event), knives
      end
      if commit_to_roles?(event)
        role_from_file files_pushed(event), knives
      end
    end
  end

  $stdout.puts "finished perform_upload_from_file: #{event['after']}"
  return # no error
end

.pluralize(n, singular, plural = nil) ⇒ Object



23
24
25
26
27
28
# File 'lib/kitchen_hooks/helpers.rb', line 23

def self.pluralize n, singular, plural=nil
  plural = "#{singular}s" if plural.nil?
  return "no #{plural}" if n.zero?
  return "1 #{singular}" if n == 1
  "#{n} #{plural}"
end

.push_details(event) ⇒ Object



529
530
531
532
533
534
# File 'lib/kitchen_hooks/helpers.rb', line 529

def self.push_details event
  return if event.nil?
  %Q|
    <a href="#{gitlab_url(event)}">#{event['after']}</a> to <a href="#{repo_url(event)}">#{repo_name(event)}</a>
    |.strip
end

.repo_name(event) ⇒ Object



551
552
553
# File 'lib/kitchen_hooks/helpers.rb', line 551

def self.repo_name event
  File::basename event['repository']['url'], '.git'
end

.repo_namespace(event) ⇒ Object



556
557
558
# File 'lib/kitchen_hooks/helpers.rb', line 556

def self.repo_namespace event
  event['project']['path_with_namespace'].split('/')[0] rescue nil
end

.repo_url(event) ⇒ Object



590
591
592
# File 'lib/kitchen_hooks/helpers.rb', line 590

def self.repo_url event
  git_daemon_style_url(event).sub(/^git/, 'http').sub(/\.git$/, '')
end

.report_error(e, msg = nil) ⇒ Object



47
48
49
50
51
52
53
# File 'lib/kitchen_hooks/helpers.rb', line 47

def self.report_error e, msg=nil
  msg = e.message if msg.nil?
  $stdout.puts msg
  $stdout.puts e.message
  $stdout.puts e.backtrace.inspect
  msg
end

.role_from_file(roles, knives) ⇒ Object



252
253
254
255
256
257
258
259
260
261
# File 'lib/kitchen_hooks/helpers.rb', line 252

def self.role_from_file roles, knives
  $stdout.puts 'Uploading roles'
  begin
    roles.each do |role|
      with_each_knife_do 'role from file ' + role, knives
    end
  rescue
    raise 'Could not upload roles'
  end
end

.role_repo?(event) ⇒ Boolean

Returns:

  • (Boolean)


584
585
586
587
# File 'lib/kitchen_hooks/helpers.rb', line 584

def self.role_repo? event
  repo_name(event) =~ /role/ ||
    repo_namespace(event) =~ /role/
end

.sync!Object



47
48
49
50
51
52
53
54
55
# File 'lib/kitchen_hooks/app.rb', line 47

def self.sync!
  return if @@sync_interval.nil?
  @@sync_worker = Thread.new do
    loop do
      sleep @@sync_interval
      process_sync
    end
  end
end

.tag_name(event) ⇒ Object



623
624
625
# File 'lib/kitchen_hooks/helpers.rb', line 623

def self.tag_name event
  tagged_commit event
end

.tagged_commit(event) ⇒ Object



617
618
619
620
# File 'lib/kitchen_hooks/helpers.rb', line 617

def self.tagged_commit event
  event['ref'] =~ %r{/tags/(.*)$}
  return $1 # First regex capture
end

.tagged_commit_to_cookbook?(event) ⇒ Boolean

Returns:

  • (Boolean)


663
664
665
666
667
# File 'lib/kitchen_hooks/helpers.rb', line 663

def self.tagged_commit_to_cookbook? event
  cookbook_repo?(event) &&
    event['ref'] =~ %r{/tags/} &&
    not_deleted?(event)
end

.tagged_commit_to_realm?(event) ⇒ Boolean

Returns:

  • (Boolean)


670
671
672
673
# File 'lib/kitchen_hooks/helpers.rb', line 670

def self.tagged_commit_to_realm? event
  tagged_commit_to_cookbook?(event) &&
    commit_to_realm?(event)
end

.tmp!(dir) ⇒ Object



28
# File 'lib/kitchen_hooks/app.rb', line 28

def self.tmp! dir ; @@tmp = dir end

.tmp_clone(event, commit_method) {|dir| ... } ⇒ Object

Yields:

  • (dir)


350
351
352
353
354
355
356
357
358
359
360
361
362
363
364
365
366
367
368
369
370
371
372
373
374
375
376
377
378
379
380
381
# File 'lib/kitchen_hooks/helpers.rb', line 350

def self.tmp_clone event, commit_method, &block
  $stdout.puts 'starting tmp_clone event=%s, commit_method=%s' % [
    event['after'], commit_method.inspect
  ]

  root = File::join tmp, SecureRandom.hex
  dir = File::join root, Time.now.to_f.to_s, cookbook_name(event)
  FileUtils.mkdir_p dir

  git_protocol = event['repository']['protocol']
  if git_protocol == 'daemon'
    git_clone_url = git_daemon_style_url(event)
  else
    git_clone_url = event['repository']["git_#{git_protocol}_url"]
  end

  repo = Git.clone git_clone_url, dir, log: $stdout

  commit = self.send(commit_method, event)

  $stdout.puts 'creating tmp_clone dir=%s, commit=%s' % [
    dir.inspect, commit.inspect
  ]

  repo.checkout commit

  yield dir

  FileUtils.rm_rf root
  FileUtils.rm_rf dir
  $stdout.puts 'finished tmp_clone'
end

.upload_environment(environment, knife) ⇒ Object



446
447
448
449
450
451
452
453
454
455
456
457
458
459
460
461
462
463
464
465
466
467
468
469
470
471
472
473
474
475
476
# File 'lib/kitchen_hooks/helpers.rb', line 446

def self.upload_environment environment, knife
  $stdout.puts 'started upload_environment environment=%s, knife=%s' % [
    environment.inspect, knife.inspect
  ]
  # Load the local environment from a JSON file
  local_environment = JSON::parse File.read(environment)
  local_environment.delete 'chef_type'
  local_environment.delete 'json_class'
  local_environment.delete 'cookbook_versions'

  # Load existing environment object on Chef server
  Celluloid.logger = nil
  ridley = Ridley::from_chef_config knife, ssl: { verify: false }
  chef_environment = ridley.environment.find(local_environment['name'])

  # Create environment object if it doesn't exist
  if chef_environment.nil?
    chef_environment = ridley.environment.create(local_environment)
  end

  # Merge the local environment into the existing object
  local_environment.each do |k, v|
    chef_environment.send "#{k}=".to_sym, v
  end

  # Make it so!
  chef_environment.save
  $stdout.puts 'finished upload_environment: %s' % environment
rescue
  raise 'Could not upload environment, check your syntax'
end

.verify_constraints(constraints, environment, knife) ⇒ Object



405
406
407
408
409
410
411
412
413
414
# File 'lib/kitchen_hooks/helpers.rb', line 405

def self.verify_constraints constraints, environment, knife
  $stdout.puts 'started verify_constraints environment=%s, knife=%s' % [
    environment.inspect, knife.inspect
  ]
  chef_environment = get_environment environment, knife
  unless constraints == chef_environment.cookbook_versions
    raise 'Environment did not match constraints'
  end
  $stdout.puts 'finished verify_constraints: %s' % environment
end


683
684
685
686
687
# File 'lib/kitchen_hooks/helpers.rb', line 683

def self.version_link event
  return unless v = event['version']
  url = version_url(event)
  'at <a href="%s">%s</a>' % [ url, v ]
end

.version_url(event) ⇒ Object



676
677
678
679
680
# File 'lib/kitchen_hooks/helpers.rb', line 676

def self.version_url event
  return unless v = event['version']
  url = git_daemon_style_url(event).sub(/^git/, 'http').sub(/\.git$/, '')
  "#{url}/commits/#{v}"
end

.with_each_knife(command, knives) ⇒ Object



388
389
390
391
392
393
394
395
396
# File 'lib/kitchen_hooks/helpers.rb', line 388

def self.with_each_knife command, knives
  knives.pmap do |k|
    cmd = "#{command} 2>&1" % { knife: Shellwords::escape(k) }
    $stdout.puts 'with_each_knife: %s' % cmd
    out = `#{cmd}`
    raise out unless $?.exitstatus.zero?
    out
  end
end

.with_each_knife_do(command, knives) ⇒ Object



384
385
386
# File 'lib/kitchen_hooks/helpers.rb', line 384

def self.with_each_knife_do command, knives
  with_each_knife "knife #{command} --config %{knife}", knives
end

Instance Method Details

#generic_details(e) ⇒ Object



516
# File 'lib/kitchen_hooks/helpers.rb', line 516

def generic_details e ; App.generic_details e end

#notification(e) ⇒ Object



479
# File 'lib/kitchen_hooks/helpers.rb', line 479

def notification e ; App.notification e end

#push_details(e) ⇒ Object



526
# File 'lib/kitchen_hooks/helpers.rb', line 526

def push_details e ; App.push_details e end